# SPDX-License-Identifier: Apache-2.0
# SPDX-FileCopyrightText: 2025 The Linux Foundation

"""
Job attribution test fixtures.

This package contains test fixtures extracted from production report data
for testing Jenkins job attribution logic.

Fixtures are sourced from:
https://github.com/lfreleng-actions/project-reporting-artifacts/tree/main/data/artifacts
"""
