/*
SPDX-License-Identifier: Apache-2.0
SPDX-FileCopyrightText: 2025 The Linux Foundation
*/

/**
 * Default Theme for Repository Reporting System
 * Phase 8: Renderer Modernization
 *
 * A clean, professional theme with good readability and print support.
 */

/* ============================================================================
   CSS Variables (Design Tokens)
   ============================================================================ */

:root {
    /* Colors */
    --color-primary: #2563eb;
    --color-primary-dark: #1e40af;
    --color-secondary: #64748b;
    --color-success: #16a34a;
    --color-warning: #ea580c;
    --color-danger: #dc2626;
    --color-info: #0891b2;

    /* Neutrals */
    --color-text: #1e293b;
    --color-text-light: #64748b;
    --color-background: #ffffff;
    --color-background-alt: #f8fafc;
    --color-border: #e2e8f0;
    --color-border-dark: #cbd5e1;

    /* Typography */
    --font-family-base: -apple-system, BlinkMacSystemFont, "Segoe UI", Roboto, "Helvetica Neue", Arial, sans-serif;
    --font-family-mono: "SF Mono", Monaco, "Cascadia Code", "Roboto Mono", Consolas, "Courier New", monospace;

    --font-size-base: 16px;
    --font-size-sm: 0.875rem;
    --font-size-lg: 1.125rem;
    --font-size-xl: 1.25rem;

    --line-height-base: 1.6;
    --line-height-heading: 1.3;

    /* Spacing */
    --space-xs: 0.25rem;
    --space-sm: 0.5rem;
    --space-md: 1rem;
    --space-lg: 1.5rem;
    --space-xl: 2rem;
    --space-2xl: 3rem;

    /* Layout */
    --container-max-width: 1400px;
    --border-radius: 0.375rem;
    --box-shadow: 0 1px 3px 0 rgba(0, 0, 0, 0.1), 0 1px 2px 0 rgba(0, 0, 0, 0.06);
    --box-shadow-lg: 0 10px 15px -3px rgba(0, 0, 0, 0.1), 0 4px 6px -2px rgba(0, 0, 0, 0.05);
}

/* ============================================================================
   Base Styles
   ============================================================================ */

* {
    box-sizing: border-box;
}

html {
    font-size: var(--font-size-base);
    line-height: var(--line-height-base);
}

body {
    margin: 0;
    padding: 0;
    font-family: var(--font-family-base);
    color: var(--color-text);
    background-color: var(--color-background);
    -webkit-font-smoothing: antialiased;
    -moz-osx-font-smoothing: grayscale;
}

/* ============================================================================
   Typography
   ============================================================================ */

h1, h2, h3, h4, h5, h6 {
    margin: 0 0 var(--space-md) 0;
    font-weight: 700;
    line-height: var(--line-height-heading);
    color: var(--color-text);
}

h1 { font-size: 2.25rem; }
h2 { font-size: 1.875rem; }
h3 { font-size: 1.5rem; }
h4 { font-size: 1.25rem; }
h5 { font-size: 1.125rem; }
h6 { font-size: 1rem; }

p {
    margin: 0 0 var(--space-md) 0;
}

a {
    color: var(--color-primary);
    text-decoration: none;
    transition: color 0.2s ease;
}

a:hover {
    color: var(--color-primary-dark);
    text-decoration: underline;
}

strong {
    font-weight: 600;
}

code, kbd, pre, samp {
    font-family: var(--font-family-mono);
    font-size: 0.875em;
}

/* ============================================================================
   Layout
   ============================================================================ */

.container {
    max-width: var(--container-max-width);
    margin: 0 auto;
    padding: 0 var(--space-lg);
}

/* Skip Link for Accessibility */
.skip-link {
    position: absolute;
    top: -40px;
    left: 0;
    background: var(--color-primary);
    color: white;
    padding: var(--space-sm) var(--space-md);
    text-decoration: none;
    z-index: 100;
}

.skip-link:focus {
    top: 0;
}

/* ============================================================================
   Page Header
   ============================================================================ */

.page-header {
    background: linear-gradient(135deg, var(--color-primary) 0%, var(--color-primary-dark) 100%);
    color: white;
    padding: var(--space-xl) 0;
    margin-bottom: var(--space-2xl);
    box-shadow: var(--box-shadow-lg);
}

.page-header h1 {
    margin: 0 0 var(--space-sm) 0;
    color: white;
}

.page-header .metadata {
    display: flex;
    gap: var(--space-lg);
    flex-wrap: wrap;
    margin-top: var(--space-md);
    font-size: var(--font-size-sm);
    opacity: 0.95;
}

.page-header .metadata p {
    margin: 0;
}

.page-header .metadata strong {
    font-weight: 600;
}

/* ============================================================================
   Page Footer
   ============================================================================ */

.page-footer {
    background-color: var(--color-background-alt);
    border-top: 1px solid var(--color-border);
    padding: var(--space-xl) 0;
    margin-top: var(--space-2xl);
    text-align: center;
    color: var(--color-text-light);
    font-size: var(--font-size-sm);
}

.page-footer p {
    margin: var(--space-xs) 0;
}

/* ============================================================================
   Main Content
   ============================================================================ */

main {
    padding: var(--space-xl) 0;
}

/* ============================================================================
   Report Sections
   ============================================================================ */

.report-section {
    margin-bottom: var(--space-2xl);
    padding-bottom: var(--space-xl);
    border-bottom: 2px solid var(--color-border);
}

.report-section:last-child {
    border-bottom: none;
}

.report-section h2 {
    margin-bottom: var(--space-lg);
    padding-bottom: var(--space-sm);
    border-bottom: 3px solid var(--color-primary);
    color: var(--color-primary-dark);
}

.subsection {
    margin-top: var(--space-xl);
}

.subsection:first-child {
    margin-top: var(--space-lg);
}

.subsection h3 {
    margin-bottom: var(--space-md);
    color: var(--color-secondary);
}

.section-description {
    color: var(--color-text-light);
    font-size: var(--font-size-sm);
    margin-bottom: var(--space-lg);
}

/* ============================================================================
   Tables
   ============================================================================ */

table {
    width: 100%;
    border-collapse: collapse;
    margin: var(--space-lg) 0;
    background-color: var(--color-background);
    border: 1px solid var(--color-border);
    border-radius: var(--border-radius);
    overflow: hidden;
    box-shadow: var(--box-shadow);
}

thead {
    background-color: var(--color-background-alt);
}

thead th {
    padding: var(--space-md);
    text-align: left;
    font-weight: 600;
    color: var(--color-text);
    border-bottom: 2px solid var(--color-border-dark);
    font-size: var(--font-size-sm);
    text-transform: uppercase;
    letter-spacing: 0.025em;
}

tbody tr {
    border-bottom: 1px solid var(--color-border);
    transition: background-color 0.2s ease;
}

tbody tr:last-child {
    border-bottom: none;
}

tbody tr:hover {
    background-color: var(--color-background-alt);
}

tbody td {
    padding: var(--space-md);
    color: var(--color-text);
}

/* Table alignment */
th.rank-col,
td.rank {
    text-align: center;
    width: 60px;
}

td.value,
td.total,
td.score {
    text-align: right;
    font-variant-numeric: tabular-nums;
}

/* Specific table styles */
.stats-table td.value {
    font-weight: 600;
    color: var(--color-primary);
}

.repo-name,
.contributor-name,
.org-name,
.job-name {
    font-family: var(--font-family-mono);
    font-size: var(--font-size-sm);
}

.email {
    color: var(--color-text-light);
    font-size: var(--font-size-sm);
}

.status {
    font-weight: 500;
}

/* ============================================================================
   Feature Matrix
   ============================================================================ */

.feature-matrix-wrapper {
    overflow-x: auto;
    margin: var(--space-lg) 0;
}

.feature-matrix {
    min-width: 100%;
}

.feature-matrix .feature-col {
    text-align: center;
    min-width: 80px;
}

.feature-matrix .feature-indicator {
    text-align: center;
    font-size: 1.25rem;
}

.feature-present {
    color: var(--color-success);
    font-weight: bold;
}

.feature-absent {
    color: var(--color-text-light);
    opacity: 0.5;
}

.feature-summary {
    margin-top: var(--space-lg);
    padding: var(--space-md);
    background-color: var(--color-background-alt);
    border-radius: var(--border-radius);
    display: flex;
    gap: var(--space-xl);
}

.feature-summary p {
    margin: 0;
}

/* ============================================================================
   Lists
   ============================================================================ */

ul, ol {
    margin: 0 0 var(--space-md) 0;
    padding-left: var(--space-xl);
}

li {
    margin-bottom: var(--space-sm);
}

.status-summary,
.state-summary {
    list-style: none;
    padding: 0;
    margin: var(--space-md) 0;
}

.status-summary li,
.state-summary li {
    padding: var(--space-sm) var(--space-md);
    background-color: var(--color-background-alt);
    border-left: 3px solid var(--color-primary);
    margin-bottom: var(--space-sm);
    border-radius: var(--border-radius);
}

.status-summary .count,
.state-summary .count {
    color: var(--color-primary);
    font-weight: 600;
}

.no-commits-list {
    list-style: none;
    padding: 0;
}

.no-commits-list li {
    padding: var(--space-sm) var(--space-md);
    background-color: var(--color-background-alt);
    margin-bottom: var(--space-sm);
    border-radius: var(--border-radius);
}

/* ============================================================================
   Badges & Indicators
   ============================================================================ */

.state-badge {
    display: inline-block;
    padding: var(--space-xs) var(--space-sm);
    background-color: var(--color-secondary);
    color: white;
    border-radius: var(--border-radius);
    font-size: var(--font-size-sm);
    font-weight: 500;
    text-transform: uppercase;
    letter-spacing: 0.025em;
}

.count {
    font-weight: 600;
    color: var(--color-primary);
}

/* ============================================================================
   Utility Classes
   ============================================================================ */

.total-count {
    margin-top: var(--space-lg);
    padding: var(--space-md);
    background-color: var(--color-background-alt);
    border-left: 4px solid var(--color-primary);
    border-radius: var(--border-radius);
}

.table-summary {
    margin-top: var(--space-md);
    text-align: right;
    font-size: var(--font-size-sm);
    color: var(--color-text-light);
}

/* ============================================================================
   Time Windows
   ============================================================================ */

.time-window {
    margin-bottom: var(--space-xl);
    padding: var(--space-lg);
    background-color: var(--color-background-alt);
    border-radius: var(--border-radius);
    border: 1px solid var(--color-border);
}

.time-window h3 {
    margin-bottom: var(--space-sm);
}

.window-duration {
    color: var(--color-text-light);
    font-size: var(--font-size-sm);
    font-weight: normal;
}

.date-range {
    color: var(--color-text-light);
    font-size: var(--font-size-sm);
    margin-bottom: var(--space-lg);
}

/* ============================================================================
   Responsive Design
   ============================================================================ */

@media (max-width: 768px) {
    :root {
        --font-size-base: 14px;
    }

    .container {
        padding: 0 var(--space-md);
    }

    .page-header h1 {
        font-size: 1.75rem;
    }

    .page-header .metadata {
        flex-direction: column;
        gap: var(--space-sm);
    }

    table {
        font-size: var(--font-size-sm);
    }

    thead th,
    tbody td {
        padding: var(--space-sm);
    }

    .feature-matrix {
        font-size: var(--font-size-sm);
    }

    .feature-summary {
        flex-direction: column;
        gap: var(--space-sm);
    }
}

/* ============================================================================
   Print Styles
   ============================================================================ */

@media print {
    * {
        -webkit-print-color-adjust: exact;
        print-color-adjust: exact;
    }

    body {
        font-size: 10pt;
        color: #000;
    }

    .no-print,
    .skip-link {
        display: none !important;
    }

    .page-header {
        background: #2563eb;
        color: #000;
        padding: 1rem 0;
        margin-bottom: 1rem;
    }

    .page-header h1 {
        color: #fff;
        font-size: 18pt;
    }

    .report-section {
        page-break-inside: avoid;
        margin-bottom: 1.5rem;
        padding-bottom: 1rem;
    }

    .report-section h2 {
        page-break-after: avoid;
        font-size: 14pt;
    }

    table {
        page-break-inside: avoid;
        font-size: 9pt;
        box-shadow: none;
    }

    thead {
        display: table-header-group;
    }

    tr {
        page-break-inside: avoid;
    }

    a {
        color: #000;
        text-decoration: none;
    }

    .page-footer {
        margin-top: 2rem;
        padding-top: 1rem;
        border-top: 1px solid #000;
    }
}

/* ============================================================================
   Accessibility
   ============================================================================ */

/* Focus styles */
:focus {
    outline: 2px solid var(--color-primary);
    outline-offset: 2px;
}

/* Reduced motion */
@media (prefers-reduced-motion: reduce) {
    *,
    *::before,
    *::after {
        animation-duration: 0.01ms !important;
        animation-iteration-count: 1 !important;
        transition-duration: 0.01ms !important;
    }
}

/* High contrast mode support */
@media (prefers-contrast: high) {
    :root {
        --color-border: #000;
        --color-border-dark: #000;
    }

    table {
        border: 2px solid #000;
    }

    thead th {
        border-bottom: 2px solid #000;
    }

    tbody tr {
        border-bottom: 1px solid #000;
    }
}
